@@ -97,6 +97,8 @@ Client::Client(Workspace* ws)
     , wrapper(None)
     , decoration(NULL)
     , bridge(new Bridge(this))
+    , m_activityUpdatesBlocked(false)
+    , m_blockedActivityUpdatesRequireTransients(false)
     , move_resize_grab_window(None)
     , move_resize_has_keyboard_grab(false)
     , m_managed(false)
@@ -1518,12 +1520,16 @@ void Client::setOnActivity(const QString &activity, bool enable)
 /**
  * set exactly which activities this client is on
  */
+#define NULL_UUID "00000000-0000-0000-0000-000000000000"
 void Client::setOnActivities(QStringList newActivitiesList)
 {
     QString joinedActivitiesList = newActivitiesList.join(",");
     joinedActivitiesList = rules()->checkActivity(joinedActivitiesList, false);
     newActivitiesList = joinedActivitiesList.split(',', QString::SkipEmptyParts);
 
+    if (newActivitiesList.contains(NULL_UUID))
+        newActivitiesList.clear(); // turn into "all"
+
     QStringList allActivities = workspace()->activityList();
     if ( newActivitiesList.isEmpty() ||
         (newActivitiesList.count() > 1 && newActivitiesList.count() == allActivities.count()) ||
@@ -1544,15 +1550,28 @@ void Client::setOnActivities(QStringList newActivitiesList)
     updateActivities(false);
 }
 
+void Client::blockActivityUpdates(bool b)
+{
+    if (b) {
+        ++m_activityUpdatesBlocked;
+    } else {
+        Q_ASSERT(m_activityUpdatesBlocked);
+        --m_activityUpdatesBlocked;
+        if (!m_activityUpdatesBlocked)
+            updateActivities(m_blockedActivityUpdatesRequireTransients);
+    }
+}
+
 /**
  * update after activities changed
  */
 void Client::updateActivities(bool includeTransients)
 {
-    /* FIXME do I need this?
-    if ( decoration != NULL )
-        decoration->desktopChange();
-        */
+    if (m_activityUpdatesBlocked) {
+        m_blockedActivityUpdatesRequireTransients |= includeTransients;
+        return;
+    }
+    m_blockedActivityUpdatesRequireTransients = false; // reset
     if (includeTransients)
         workspace()->updateOnAllActivitiesOfTransients(this);
     workspace()->updateFocusChains(this, Workspace::FocusChainMakeFirst);
@@ -2380,6 +2399,12 @@ void Client::checkActivities()
     }
 
     newActivitiesList = QString(prop).split(',');
+
+    if (newActivitiesList.contains(NULL_UUID)) {
+        setOnAllActivities(true);
+        return;
+    }
+
     if (newActivitiesList == activityList)
         return; //expected change, it's ok.
 
