@@ -25,6 +25,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtDeclarative/QDeclarativeEngine>
 #include <QtDeclarative/QDeclarativeItem>
 #include <QtGui/QGraphicsView>
+#include <QPaintEngine>
 
 #include <KConfig>
 #include <KConfigGroup>
@@ -158,11 +159,8 @@ bool AuroraeFactory::reset(unsigned long changed)
     const QString themeName = group.readEntry("ThemeName", "example-deco");
     const KConfig config("aurorae/themes/" + themeName + '/' + themeName + "rc", KConfig::FullConfig, "data");
     const KConfigGroup themeGroup(&conf, themeName);
-    if (themeName != m_theme->themeName()) {
-        delete m_engine;
-        m_engine = new QDeclarativeEngine(this);
-        delete m_component;
-        m_component = new QDeclarativeComponent(m_engine, this);
+    if (themeName != m_themeName) {
+        m_engine->clearComponentCache();
         init();
         // recreate all decorations
         return true;
@@ -576,6 +574,18 @@ void AuroraeClient::slotAlphaChanged()
     }
 }
 
+bool AuroraeClient::animationsSupported() const
+{
+    if (!compositingActive()) {
+        return false;
+    }
+    QPixmap pix(1,1);
+    QPainter p(&pix);
+    const bool raster = p.paintEngine()->type() == QPaintEngine::Raster;
+    p.end();
+    return raster;
+}
+
 } // namespace Aurorae
 
 extern "C"
