@@ -21,6 +21,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "trackmouse.h"
 
+// KConfigSkeleton
+#include "trackmouseconfig.h"
+
 #include <QTime>
 #include <QMatrix4x4>
 
@@ -32,7 +35,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <kaction.h>
 #include <kactioncollection.h>
-#include <KDE/KConfigGroup>
 #include <KDE/KLocale>
 
 #include <math.h>
@@ -55,7 +57,7 @@ TrackMouseEffect::TrackMouseEffect()
     if ( effects->compositingType() == XRenderCompositing)
         m_angleBase = 1.57079632679489661923; // Pi/2
 #endif
-    if ( effects->compositingType() == OpenGLCompositing)
+    if ( effects->isOpenGLCompositing())
         m_angleBase = 90.0;
     m_mousePolling = false;
     KActionCollection *actionCollection = new KActionCollection(this);
@@ -84,14 +86,14 @@ TrackMouseEffect::~TrackMouseEffect()
 void TrackMouseEffect::reconfigure(ReconfigureFlags)
 {
     m_modifiers = 0;
-    KConfigGroup conf = effects->effectConfig("TrackMouse");
-    if (conf.readEntry("Shift", false))
+    TrackMouseConfig::self()->readConfig();
+    if (TrackMouseConfig::shift())
         m_modifiers |= Qt::ShiftModifier;
-    if (conf.readEntry("Alt", false))
+    if (TrackMouseConfig::alt())
         m_modifiers |= Qt::AltModifier;
-    if (conf.readEntry("Control", true))
+    if (TrackMouseConfig::control())
         m_modifiers |= Qt::ControlModifier;
-    if (conf.readEntry("Meta", true))
+    if (TrackMouseConfig::meta())
         m_modifiers |= Qt::MetaModifier;
 
     if (m_modifiers) {
@@ -122,12 +124,11 @@ void TrackMouseEffect::paintScreen(int mask, QRegion region, ScreenPaintData& da
     if (!m_active)
         return;
 
-    if ( effects->compositingType() == OpenGLCompositing && m_texture[0] && m_texture[1]) {
-        GLShader *shader(0);
+    if ( effects->isOpenGLCompositing() && m_texture[0] && m_texture[1]) {
+        ShaderBinder binder(ShaderManager::GenericShader);
+        GLShader *shader(binder.shader());
         QMatrix4x4 modelview;
-        if (ShaderManager::instance()->isValid()) {
-            ShaderManager::instance()->pushShader(ShaderManager::GenericShader);
-            shader = ShaderManager::instance()->getBoundShader();
+        if (shader) {
             modelview = shader->getUniformMatrix4x4("modelview");
         }
         glEnable(GL_BLEND);
@@ -142,7 +143,6 @@ void TrackMouseEffect::paintScreen(int mask, QRegion region, ScreenPaintData& da
                 shader->setUniform(GLShader::ModelViewMatrix, matrix);
                 shader->setUniform(GLShader::Saturation, 1.0);
                 shader->setUniform(GLShader::ModulationConstant, QVector4D(1.0, 1.0, 1.0, 1.0));
-                shader->setUniform(GLShader::AlphaToOne, 0);
             } else
                 pushMatrix(matrix);
             m_texture[i]->bind();
@@ -152,9 +152,8 @@ void TrackMouseEffect::paintScreen(int mask, QRegion region, ScreenPaintData& da
                 popMatrix();
         }
         glDisable(GL_BLEND);
-        if (ShaderManager::instance()->isValid()) {
+        if (shader) {
             shader->setUniform(GLShader::ModelViewMatrix, modelview);
-            ShaderManager::instance()->popShader();
         }
     }
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
@@ -248,7 +247,7 @@ void TrackMouseEffect::loadTexture()
         return;
 
     for (int i = 0; i < 2; ++i) {
-        if ( effects->compositingType() == OpenGLCompositing) {
+        if ( effects->isOpenGLCompositing()) {
             QImage img(f[i]);
             m_texture[i] = new GLTexture(img);
             m_lastRect[i].setSize(img.size());
