@@ -37,6 +37,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "overlaywindow.h"
 #include "rules.h"
 #include "unmanaged.h"
+#include "useractions.h"
 #include "effects.h"
 
 #include <QWhatsThis>
@@ -50,6 +51,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <X11/Xatom.h>
 #include <QX11Info>
 
+#include "composite.h"
+
 namespace KWin
 {
 
@@ -440,20 +443,20 @@ bool Workspace::workspaceEvent(XEvent * e)
     case Expose:
         if (compositing()
                 && (e->xexpose.window == rootWindow()   // root window needs repainting
-                    || (scene->overlayWindow()->window() != None && e->xexpose.window == scene->overlayWindow()->window()))) { // overlay needs repainting
-            addRepaint(e->xexpose.x, e->xexpose.y, e->xexpose.width, e->xexpose.height);
+                    || (m_compositor->overlayWindow() != None && e->xexpose.window == m_compositor->overlayWindow()))) { // overlay needs repainting
+            m_compositor->addRepaint(e->xexpose.x, e->xexpose.y, e->xexpose.width, e->xexpose.height);
         }
         break;
     case VisibilityNotify:
-        if (compositing() && scene->overlayWindow()->window() != None && e->xvisibility.window == scene->overlayWindow()->window()) {
-            bool was_visible = scene->overlayWindow()->isVisible();
-            scene->overlayWindow()->setVisibility((e->xvisibility.state != VisibilityFullyObscured));
-            if (!was_visible && scene->overlayWindow()->isVisible()) {
+        if (compositing() && m_compositor->overlayWindow() != None && e->xvisibility.window == m_compositor->overlayWindow()) {
+            bool was_visible = m_compositor->isOverlayWindowVisible();
+            m_compositor->setOverlayWindowVisibility((e->xvisibility.state != VisibilityFullyObscured));
+            if (!was_visible && m_compositor->isOverlayWindowVisible()) {
                 // hack for #154825
-                addRepaintFull();
-                QTimer::singleShot(2000, this, SLOT(addRepaintFull()));
+                m_compositor->addRepaintFull();
+                QTimer::singleShot(2000, m_compositor, SLOT(addRepaintFull()));
             }
-            checkCompositeTimer();
+            m_compositor->scheduleRepaint();
         }
         break;
     default:
@@ -463,8 +466,8 @@ bool Workspace::workspaceEvent(XEvent * e)
                 // desktopResized() should take care of when the size or
                 // shape of the desktop has changed, but we also want to
                 // catch refresh rate changes
-                if (xrrRefreshRate != currentRefreshRate())
-                    compositeResetTimer.start(0);
+                if (m_compositor->xrrRefreshRate() != currentRefreshRate())
+                    m_compositor->setCompositeResetTimer(0);
             }
 
         } else if (e->type == Extensions::syncAlarmNotifyEvent() && Extensions::syncAvailable()) {
@@ -879,7 +882,7 @@ void Client::enterNotifyEvent(XCrossingEvent* e)
         }
 #undef MOUSE_DRIVEN_FOCUS
 
-        if (options->focusPolicy() == Options::ClickToFocus || workspace()->windowMenuShown())
+        if (options->focusPolicy() == Options::ClickToFocus || workspace()->userActionsMenu()->isShown())
             return;
 
         QPoint currentPos(e->x_root, e->y_root);
@@ -1177,16 +1180,17 @@ bool Client::buttonPressEvent(Window w, int button, int state, int x, int y, int
         XAllowEvents(display(), ReplayPointer, CurrentTime);  //xTime());
         return true;
     }
-    if (w == decorationId() || w == inputId()) {
-        if (w == inputId()) {
-            x = x_root - geometry().x() + padding_left;
-            y = y_root - geometry().y() + padding_top;
-        }
+    if (w == inputId()) {
+        x = x_root - geometry().x() + padding_left;
+        y = y_root - geometry().y() + padding_top;
+        // New API processes core events FIRST and only passes unused ones to the decoration
+        return processDecorationButtonPress(button, state, x, y, x_root, y_root, true);
+    }
+    if (w == decorationId()) {
         if (dynamic_cast<KDecorationUnstable*>(decoration))
             // New API processes core events FIRST and only passes unused ones to the decoration
             return processDecorationButtonPress(button, state, x, y, x_root, y_root, true);
-        else
-            return false; // Don't eat old API decoration events
+        return false;
     }
     if (w == frameId())
         processDecorationButtonPress(button, state, x, y, x_root, y_root);
@@ -1341,9 +1345,9 @@ void Client::checkQuickTilingMaximizationZones(int xroot, int yroot)
         }
 
         if (mode != QuickTileNone) {
-            if (yroot <= area.y() + area.height() / 4)
+            if (yroot <= area.y() + area.height() * options->electricBorderCornerRatio())
                 mode |= QuickTileTop;
-            else if (yroot >= area.y() + area.height() - area.height() / 4)
+            else if (yroot >= area.y() + area.height() - area.height()  * options->electricBorderCornerRatio())
                 mode |= QuickTileBottom;
         } else if (options->electricBorderMaximize() && yroot <= area.y() + 5 && isMaximizable())
             mode = QuickTileMaximize;
