@@ -28,6 +28,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kdemacros.h>
 
 #include <X11/Xlib.h>
+#include <X11/Xlib-xcb.h>
 #include <fixx11h.h>
 
 #include <kwinconfig.h>
@@ -40,8 +41,23 @@ namespace KWin
 
 enum CompositingType {
     NoCompositing = 0,
-    OpenGLCompositing,
-    XRenderCompositing
+    /**
+     * Used as a flag whether OpenGL based compositing is used.
+     * The flag is or-ed to the enum values of the specific OpenGL types.
+     * The actual Compositors use the @c OpenGL1Compositing or @c OpenGL2Compositing
+     * flags. If you need to know whether OpenGL is used, either and the flag or
+     * use EffectsHandler::isOpenGLCompositing().
+     **/
+    OpenGLCompositing = 1,
+    XRenderCompositing = 1<<1,
+    OpenGL1Compositing = 1<<2 | OpenGLCompositing,
+    OpenGL2Compositing = 1<<3 | OpenGLCompositing
+};
+
+enum OpenGLPlatformInterface {
+    NoOpenGLPlatformInterface = 0,
+    GlxPlatformInterface,
+    EglPlatformInterface
 };
 
 enum clientAreaOption {
@@ -117,6 +133,12 @@ KWIN_EXPORT Display* display()
     return QX11Info::display();
 }
 
+inline
+KWIN_EXPORT xcb_connection_t *connection()
+{
+    return XGetXCBConnection(display());
+}
+
 inline
 KWIN_EXPORT Window rootWindow()
 {
