@@ -58,7 +58,7 @@ void KWIN_EXPORT initGLX();
 // Initializes OpenGL stuff. This includes resolving function pointers as
 //  well as checking for GL version and extensions
 //  Note that GL context has to be created by the time this function is called
-void KWIN_EXPORT initGL();
+void KWIN_EXPORT initGL(OpenGLPlatformInterface platformInterface);
 // Initializes EGL function pointers
 void KWIN_EXPORT initEGL();
 // Cleans up all resources hold by the GL Context
@@ -190,7 +190,7 @@ public:
     };
 
     enum IntUniform {
-        AlphaToOne,
+        AlphaToOne,     ///< @deprecated no longer used
         IntUniformCount
     };
 
@@ -204,6 +204,7 @@ protected:
     GLShader();
     bool loadFromFiles(const QString& vertexfile, const QString& fragmentfile);
     bool load(const QByteArray &vertexSource, const QByteArray &fragmentSource);
+    const QByteArray prepareSource(GLenum shaderType, const QByteArray &sourceCode) const;
     bool compile(GLuint program, GLenum shaderType, const QByteArray &sourceCode) const;
     void bind();
     void unbind();
@@ -219,6 +220,9 @@ private:
     int mFloatLocation[FloatUniformCount];
     int mIntLocation[IntUniformCount];
 
+    static bool sColorCorrect;
+
+    friend class ColorCorrection;
     friend class ShaderManager;
 };
 
@@ -247,9 +251,6 @@ public:
          * normalized texture coordinates. Defaults to @c (1.0/1.0). And expects a @c vec3
          * uniform @c colorManiuplation, with @c x being opacity, @c y being brightness and
          * @c z being saturation. All three values default to @c 1.0.
-         * The fragment shader takes a uniform @c u_forceAlpha to force the alpha component
-         * to @c 1.0 if it is set to a value not @c 0. This uniform is useful when rendering
-         * a RGB-only window and the alpha channel is required in a later step.
          * The sampler uniform is @c sample and defaults to @c 0.
          * The shader uses two vertex attributes @c vertex and @c texCoord.
          **/
@@ -283,6 +284,13 @@ public:
      **/
     bool isShaderBound() const;
     /**
+     * Allows to query whether Shaders are supported by the compositor, that is
+     * whether the Shaders compiled successfully.
+     *
+     * With OpenGL 1 compositing this method will always return @c false.
+     *
+     * Do not use this method to check whether the compositor uses OpenGL 1 or 2,
+     * use @link EffectsHandler::compositingType instead.
      * @return @c true if the built-in shaders are valid, @c false otherwise
      **/
     bool isValid() const;
@@ -355,6 +363,15 @@ public:
      * @return a pointer to the ShaderManager instance
      **/
     static ShaderManager *instance();
+    /**
+     * @brief Ensures that the ShaderManager is disabled.
+     *
+     * Used only by an OpenGL 1 Scene which does not use OpenGL 2 Shaders.
+     *
+     * @internal
+     * @since 4.10
+     **/
+    static void disable();
 
     /**
      * @internal
@@ -378,6 +395,95 @@ private:
     static ShaderManager *s_shaderManager;
 };
 
+/**
+ * An helper class to push a Shader on to ShaderManager's stack and ensuring that the Shader
+ * gets popped again from the stack automatically once the object goes out of life.
+ *
+ * How to use:
+ * @code
+ * {
+ * GLShader *myCustomShaderIWantToPush;
+ * ShaderBinder binder(myCustomShaderIWantToPush);
+ * // do stuff with the shader being pushed on the stack
+ * }
+ * // here the Shader is automatically popped as helper does no longer exist.
+ * @endcode
+ *
+ * This class takes care for the case that the Compositor uses OpenGL 1 and the ShaderManager is
+ * not valid. In that case the helper does not do anything. So this helper can be used to simplify
+ * the code to remove checks for OpenGL 1/2.
+ * @since 4.10
+ **/
+class KWIN_EXPORT ShaderBinder
+{
+public:
+    /**
+     * @brief Pushes the Shader of the given @p type to the ShaderManager's stack.
+     *
+     * @param type The built-in Shader type
+     * @param reset Whether all uniforms should be reset to their default values. Defaults to false.
+     * @see ShaderManager::pushShader
+     **/
+    ShaderBinder(ShaderManager::ShaderType type, bool reset = false);
+    /**
+     * @brief Pushes the given @p shader to the ShaderManager's stack.
+     *
+     * @param shader The Shader to push on the stack
+     * @see ShaderManager::pushShader
+     **/
+    ShaderBinder(GLShader *shader);
+    ~ShaderBinder();
+
+    /**
+     * @return The Shader pushed to the Stack. On OpenGL 1 this returns a @c null pointer.
+     **/
+    GLShader *shader();
+
+private:
+    GLShader *m_shader;
+};
+
+inline
+ShaderBinder::ShaderBinder(ShaderManager::ShaderType type, bool reset)
+    : m_shader(NULL)
+{
+#ifdef KWIN_HAVE_OPENGL_1
+    if (!ShaderManager::instance()->isValid()) {
+        return;
+    }
+#endif
+    m_shader = ShaderManager::instance()->pushShader(type, reset);
+}
+
+inline
+ShaderBinder::ShaderBinder(GLShader *shader)
+    : m_shader(shader)
+{
+#ifdef KWIN_HAVE_OPENGL_1
+    if (!ShaderManager::instance()->isValid()) {
+        return;
+    }
+#endif
+    ShaderManager::instance()->pushShader(shader);
+}
+
+inline
+ShaderBinder::~ShaderBinder()
+{
+#ifdef KWIN_HAVE_OPENGL_1
+    if (!ShaderManager::instance()->isValid()) {
+        return;
+    }
+#endif
+    ShaderManager::instance()->popShader();
+}
+
+inline
+GLShader* ShaderBinder::shader()
+{
+    return m_shader;
+}
+
 /**
  * @short Render target object
  *
