@@ -681,7 +681,7 @@ void DesktopCorona::checkActivities()
 void DesktopCorona::currentActivityChanged(const QString &newActivity)
 {
     kDebug() << newActivity;
-    Activity *act =activity(newActivity);
+    Activity *act = activity(newActivity);
     if (act) {
         act->ensureActive();
     }
@@ -696,6 +696,11 @@ Activity* DesktopCorona::activity(const QString &id)
     return m_activities.value(id);
 }
 
+KActivities::Controller *DesktopCorona::activityController()
+{
+    return m_activityController;
+}
+
 void DesktopCorona::activityAdded(const QString &id)
 {
     //TODO more sanity checks
