@@ -201,7 +201,7 @@ QString Core::checkBatteryStatus(bool notify)
                                    i.value(), i.key());
             }
             if (notify) {
-                emitNotification("brokenbattery", lastMessage, "dialog-warning");
+                emitRichNotification("brokenbattery", i18n("Broken Battery"), lastMessage);
             }
         }
     }
@@ -222,7 +222,7 @@ QString Core::checkBatteryStatus(bool notify)
                                "verify if your battery is faulted.", notice.vendor, notice.url, notice.batteryId);
         }
         if (notify) {
-            emitNotification("brokenbattery", lastMessage, "dialog-warning");
+            emitRichNotification("brokenbattery", i18n("Check Your Battery"), lastMessage);
         }
     }
 
@@ -344,7 +344,7 @@ void Core::loadProfile(bool force)
     if (!config.isValid()) {
         emitNotification("powerdevilerror", i18n("The profile \"%1\" has been selected, "
                          "but it does not exist.\nPlease check your PowerDevil configuration.",
-                         profileId), "dialog-error");
+                         profileId));
         return;
     }
 
@@ -380,7 +380,7 @@ void Core::loadProfile(bool force)
                 emitNotification("powerdevilerror", i18n("The profile \"%1\" tried to activate %2, "
                                 "a non existent action. This is usually due to an installation problem"
                                 " or to a configuration problem.",
-                                profileId, actionName), "dialog-warning");
+                                profileId, actionName));
             }
         }
 
@@ -453,15 +453,25 @@ void Core::onDeviceAdded(const QString& udi)
     }
 
     if (!connect(b, SIGNAL(chargePercentChanged(int,QString)),
-                 this, SLOT(onBatteryChargePercentChanged(int,QString)))) {
+                 this, SLOT(onBatteryChargePercentChanged(int,QString))) ||
+        !connect(b, SIGNAL(chargeStateChanged(int,QString)),
+                 this, SLOT(onBatteryChargeStateChanged(int,QString)))) {
         emitNotification("powerdevilerror", i18n("Could not connect to battery interface.\n"
-                         "Please check your system configuration"), "dialog-error");
+                         "Please check your system configuration"));
     }
 
     kDebug() << "A new battery was detected";
 
     m_batteriesPercent[udi] = b->chargePercent();
+    if (b->chargeState() == Solid::Battery::NoCharge) {
+      m_batteriesCharged[udi] = true;
+    } else {
+      m_batteriesCharged[udi] = false;
+    }
     m_loadedBatteriesUdi.append(udi);
+
+    // So we get a "Battery is low" notification directly on system startup if applicable
+    emitBatteryChargePercentNotification(b->chargePercent(), 100);
 }
 
 void Core::onDeviceRemoved(const QString& udi)
@@ -477,19 +487,73 @@ void Core::onDeviceRemoved(const QString& udi)
 
     disconnect(b, SIGNAL(chargePercentChanged(int,QString)),
                this, SLOT(onBatteryChargePercentChanged(int,QString)));
+    disconnect(b, SIGNAL(chargeStateChanged(int,QString)),
+               this, SLOT(onBatteryChargeStateChanged(int,QString)));
 
     kDebug() << "An existing battery has been removed";
 
     m_batteriesPercent.remove(udi);
+    m_batteriesCharged.remove(udi);
     m_loadedBatteriesUdi.removeOne(udi);
 }
 
 void Core::emitNotification(const QString &evid, const QString &message, const QString &iconname)
 {
-    KNotification::event(evid, message, KIcon(iconname).pixmap(20, 20),
+    if (!iconname.isEmpty()) {
+      KNotification::event(evid, message, KIcon(iconname).pixmap(48,48),
+                           0, KNotification::CloseOnTimeout, m_applicationData);
+    } else {
+      KNotification::event(evid, message, QPixmap(),
+                           0, KNotification::CloseOnTimeout, m_applicationData);
+    }
+}
+
+void Core::emitRichNotification(const QString &evid, const QString &title, const QString &message)
+{
+    KNotification::event(evid, title, message, QPixmap(),
                          0, KNotification::CloseOnTimeout, m_applicationData);
 }
 
+bool Core::emitBatteryChargePercentNotification(int currentPercent, int previousPercent)
+{
+    if (m_backend->acAdapterState() == BackendInterface::Plugged) {
+        return false;
+    }
+
+    if (currentPercent <= PowerDevilSettings::batteryCriticalLevel() &&
+        previousPercent > PowerDevilSettings::batteryCriticalLevel()) {
+        switch (PowerDevilSettings::batteryCriticalAction()) {
+        case 3:
+            emitRichNotification("criticalbattery", i18n("Battery Critical (%1% Remaining)", currentPercent),
+                             i18n("Your battery level is critical, the computer will be halted in 30 seconds."));
+            m_criticalBatteryTimer->start();
+            break;
+        case 2:
+            emitRichNotification("criticalbattery", i18n("Battery Critical (%1% Remaining)", currentPercent),
+                             i18n("Your battery level is critical, the computer will be hibernated in 30 seconds."));
+            m_criticalBatteryTimer->start();
+            break;
+        case 1:
+            emitRichNotification("criticalbattery", i18n("Battery Critical (%1% Remaining)", currentPercent),
+                             i18n("Your battery level is critical, the computer will be suspended in 30 seconds."));
+            m_criticalBatteryTimer->start();
+            break;
+        default:
+            emitRichNotification("criticalbattery", i18n("Battery Critical (%1% Remaining)", currentPercent),
+                                 i18n("Your battery level is critical, save your work as soon as possible."));
+            break;
+        }
+        return true;
+    } else if (currentPercent <= PowerDevilSettings::batteryLowLevel() &&
+               previousPercent > PowerDevilSettings::batteryLowLevel()) {
+        emitRichNotification("lowbattery", i18n("Battery Low (%1% Remaining)", currentPercent),
+                             i18n("Your battery is low. If you need to continue using your computer, either plug in your computer, or shut it down and then change the battery."));
+        return true;
+    }
+    return false;
+}
+
+
 void Core::onAcAdapterStateChanged(PowerDevil::BackendInterface::AcAdapterState state)
 {
     kDebug();
@@ -501,13 +565,14 @@ void Core::onAcAdapterStateChanged(PowerDevil::BackendInterface::AcAdapterState
         // If the AC Adaptor has been plugged in, let's clear some pending suspend actions
         if (m_criticalBatteryTimer->isActive()) {
             m_criticalBatteryTimer->stop();
-            emitNotification("criticalbattery",
-                             i18n("The power adaptor has been plugged in – all pending suspend actions have been canceled."));
+            emitRichNotification("criticalbattery",
+                             i18n("AC Adapter Plugged In"),
+                             i18n("All pending suspend actions have been canceled."));
         } else {
-            emitNotification("pluggedin", i18n("The power adaptor has been plugged in."));
+            emitRichNotification("pluggedin", i18n("Running on AC power"), i18n("The power adaptor has been plugged in."));
         }
     } else {
-        emitNotification("unplugged", i18n("The power adaptor has been unplugged."));
+        emitRichNotification("unplugged", i18n("Running on Battery Power"), i18n("The power adaptor has been unplugged."));
     }
 }
 
@@ -515,7 +580,7 @@ void Core::onBackendError(const QString& error)
 {
     emitNotification("powerdevilerror", i18n("KDE Power Management System could not be initialized. "
                          "The backend reported the following error: %1\n"
-                         "Please check your system configuration", error), "dialog-error");
+                         "Please check your system configuration", error));
 }
 
 void Core::onBatteryChargePercentChanged(int percent, const QString &udi)
@@ -530,42 +595,38 @@ void Core::onBatteryChargePercentChanged(int percent, const QString &udi)
     // Update the battery percentage
     m_batteriesPercent[udi] = percent;
 
-    // And check if we need to do stuff
-    if (m_backend->acAdapterState() == BackendInterface::Plugged) {
+    if (currentPercent < previousPercent) {
+        if (emitBatteryChargePercentNotification(currentPercent, previousPercent)) {
+            // Only refresh status if a notification has actually been emitted
+            refreshStatus();
+        }
+    }
+}
+
+void Core::onBatteryChargeStateChanged(int state, const QString &udi)
+{
+    bool previousCharged = true;
+    for (QHash<QString,bool>::const_iterator i = m_batteriesCharged.constBegin(); i != m_batteriesCharged.constEnd(); ++i) {
+        if (i.value() != Solid::Battery::NoCharge) {
+            previousCharged = false;
+            break;
+        }
+    }
+
+    bool currentCharged = previousCharged;
+    if (state != Solid::Battery::NoCharge) {
+        currentCharged = false;
+        m_batteriesCharged[udi] = false;
+    } else {
+        m_batteriesCharged[udi] = true;
+    }
+
+    if (m_backend->acAdapterState() != BackendInterface::Plugged) {
         return;
     }
 
-    if (currentPercent <= PowerDevilSettings::batteryCriticalLevel() &&
-        previousPercent > PowerDevilSettings::batteryCriticalLevel()) {
-        switch (PowerDevilSettings::batteryCriticalAction()) {
-        case 3:
-            emitNotification("criticalbattery",
-                             i18n("Your battery level is critical, the computer will be halted in 30 seconds."),
-                             "dialog-warning");
-            m_criticalBatteryTimer->start();
-            break;
-        case 2:
-            emitNotification("criticalbattery",
-                             i18n("Your battery level is critical, the computer will be hibernated in 30 seconds."),
-                             "dialog-warning");
-            m_criticalBatteryTimer->start();
-            break;
-        case 1:
-            emitNotification("criticalbattery",
-                             i18n("Your battery level is critical, the computer will be suspended in 30 seconds."),
-                             "dialog-warning");
-            m_criticalBatteryTimer->start();
-            break;
-        default:
-            emitNotification("criticalbattery",
-                             i18n("Your battery level is critical: save your work as soon as possible."),
-                             "dialog-warning");
-            break;
-        }
-    } else if (currentPercent <= PowerDevilSettings::batteryLowLevel() &&
-               previousPercent > PowerDevilSettings::batteryLowLevel()) {
-        emitNotification("lowbattery", i18n("Your battery has reached a low level."),
-                         "dialog-warning");
+    if (previousCharged == false && currentCharged == true) {
+        emitRichNotification("fullbattery", i18n("Charge Complete"), i18n("Your battery is now fully charged."));
         refreshStatus();
     }
 }
